import httpx
from typing import Any, Optional, Dict


class APIErrorResponse:
    def __init__(self, message: str, status_code: int):
        self.message = message
        self.status_code = status_code

    def to_dict(self) -> Dict[str, Any]:
        return {"error": self.message, "status_code": self.status_code}

    def __repr__(self) -> str:
        return f"<APIErrorResponse(status_code={self.status_code}, message='{self.message}')>"


class APIResponse:
    def __init__(self, response: httpx.Response):
        self._response = response
        self.status_code = response.status_code
        self.headers = response.headers
        self._json = self._parse_json(response)

    def _parse_json(self, response: httpx.Response) -> Optional[Any]:
        """Parses the JSON content of the response."""
        try:
            return response.json()
        except ValueError:
            return APIErrorResponse("Invalid JSON response", response.status_code)

    @property
    def json(self) -> Optional[Any]:
        """Returns the JSON content of the response or an APIErrorResponse if parsing fails."""
        return self._json

    def is_success(self) -> bool:
        """Returns True if the response status code indicates success."""
        return 200 <= self.status_code < 300

    def raise_for_status(self):
        """Raises an HTTPError if the response status code indicates an error."""
        if not self.is_success():
            raise httpx.HTTPStatusError(
                f"HTTP Error {self.status_code} for url {self._response.url}",
                request=self._response.request,
                response=self._response,
            )

    def __repr__(self) -> str:
        return (
            f"<APIResponse(status_code={self.status_code}, "
            f"headers={dict(self.headers)}, "
            f"json={self._json})>"
        )



from ._client import Bohrium, AsyncBohrium
from ._base_client import AsyncAPIClient, SyncAPIClient, BaseClient
from ._utils._logs import setup_logging as _setup_logging

_setup_logging()

__all__ = [
    "Bohrium",
    "AsyncBohrium",
    "AsyncAPIClient",
    "SyncAPIClient",
    "BaseClient",
]



from __future__ import annotations

import os
from typing import Mapping, Optional, Union

from httpx import URL, Client, Response, Timeout
from typing_extensions import override

from . import _exceptions, resources
from ._base_client import DEFAULT_MAX_RETRIES, AsyncAPIClient, SyncAPIClient
from ._exceptions import APIStatusError, BohriumError
from ._version import __version__


class Bohrium(SyncAPIClient):
    job: resources.Job

    # client options
    access_key: str
    project_id: Union[str, None]

    def __init__(
        self,
        access_key: Optional[str] = None,
        base_url: Optional[Union[str, URL]] = None,
        project_id: Optional[str] = None,
        timeout: Optional[Union[float, Timeout]] = 30.0,
        max_retries: Optional[int] = DEFAULT_MAX_RETRIES,
        http_client: Optional[Client] = None,
    ) -> None:
        """Construct a new synchronous openai client instance."""
        if access_key is None:
            access_key = os.environ.get("BOHRIUM_ACCESS_KEY")
        if access_key is None:
            raise BohriumError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ACCESS_KEY environment variable"
            )
        self.access_key = access_key
        self.params = {"accessKey": self.access_key}
        if project_id is None:
            project_id = os.environ.get("BOHRIUM_PROJECT_ID")

        if project_id is None:
            raise BohriumError(
                "The project_id client option must be set either by passing project_id to the client or by setting the BOHRIUM_PROJECT_ID environment variable"
            )

        self.project_id = project_id

        if base_url is None:
            base_url = os.environ.get("BOHRIUM_BASE_URL")

        if base_url is None:
            base_url = "https://openapi.dp.tech"

        super().__init__(
            _version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=self.default_headers,
        )

        self.job = resources.Job(self)

    @property
    @override
    def default_headers(self) -> dict[str, str]:
        return {
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.access_key}",
            "x-app-key": "agent",
        }

    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: Response,
    ) -> APIStatusError:
        data = body.get("error", body) if isinstance(body, Mapping) else body
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=data)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(
                err_msg, response=response, body=data
            )

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(
                err_msg, response=response, body=data
            )

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=data)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=data)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(
                err_msg, response=response, body=data
            )

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=data)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(
                err_msg, response=response, body=data
            )
        return APIStatusError(err_msg, response=response, body=data)


class AsyncBohrium(AsyncAPIClient):
    pass



import os

from setuptools import find_packages, setup


def read_file(filename):
    with open(os.path.join(os.path.dirname(__file__), filename), encoding="utf-8") as f:
        long_description = f.read()
    return long_description


setup(
    name="bohrium-sdk",
    version="0.7.0",
    author="dingzhaohan",
    author_email="dingzh@dp.tech",
    url="https://github.com/dingzhaohan",
    description="bohrium openapi python sdk",
    long_description=read_file("README.md"),  # detail description for pypi
    long_description_content_type="text/markdown",  # file format
    packages=find_packages("src", exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    package_dir={"": "src"},  # find_packages define code directory
    package_data={
        # include .txt all of them
        "": ["*.txt"]
    },
    install_requires=[],
    python_requires=">=3.7",
    entry_points={},
)



import logging
import os
import uuid
from pathlib import Path
import humps
# from ..._resource import BaseClient
from pprint import pprint
from typing import Optional

from ..._resource import AsyncAPIResource, SyncAPIResource
from ..._response import APIResponse
from ...types.job.job import JobAddRequest
from ..tiefblue.tiefblue import Tiefblue


log = logging.getLogger(__name__)


class Job(SyncAPIResource):

    def create(self, project_id, name='', group_id=0):
        data = {
            'projectId': project_id
        }
        if name:
            data['name'] = name
        if group_id:
            data['bohrGroupId'] = group_id
        try:
            data = self._client.post(f'/openapi/v1/job/create', json=data, params=self._client.params)
            data = data.json()
        except Exception as e:
            raise e
        return data.get("data", {})
    
    def detail(self, job_id):
        log.info(f"detail job {job_id}")
        response = self._client.get(f"/openapi/v1/job/{job_id}")

        log.info(response.json())
        log.debug(response)
        return APIResponse(response).json.get("data")
        #return response.json().get("data")

    def submit(
        self,
        project_id: int,
        job_name: str,
        machine_type: str,
        cmd: str,
        image_address: str,
        job_group_id: int = 0,
        work_dir: str = "",
        result: str = "",
        dataset_path: list = [],
        log_files: list = [],
        out_files: list = [],
    ):
        # log.info(f"submit job {name},project_id:{project_id}")
        data = self.create_job(project_id, job_name, job_group_id)
        print(data)
        if work_dir != "":
            if not os.path.exists(work_dir):
                raise FileNotFoundError
            if os.path.isdir(work_dir):
                self.uploadr(work_dir, data["storePath"], data["token"])
            else:
                file_name = os.path.basename(work_dir)
                object_key = os.path.join(data["storePath"], file_name)
                self.upload(work_dir, object_key, data["token"])

        ep = os.path.expanduser(result)
        p = Path(ep).absolute().resolve()
        p = p.joinpath(str(uuid.uuid4()) + "_temp.zip")

        job_add_request = JobAddRequest(
            download_path=str(p.absolute().resolve()),
            dataset_path=dataset_path,
            job_name=job_name,
            project_id=project_id,
            job_id=data["jobId"],
            oss_path=data["storePath"],
            image_name=image_address,
            scass_type=machine_type,
            cmd=cmd,
            log_files=log_files,
            out_files=out_files,
        )
        return self.insert(job_add_request.to_dict())

    def insert(self, **kwargs):
        camel_data = {humps.camelize(k): v for k, v in kwargs.items()}
        if not isinstance(camel_data['ossPath'], list):
            camel_data['ossPath'] = [camel_data['ossPath']]
        if 'logFile' in camel_data:
            camel_data['logFiles'] = camel_data['logFile']
        if 'logFiles' in camel_data and not isinstance(camel_data['logFiles'], list):
            camel_data['logFiles'] = [camel_data['logFiles']]
        response = self._client.post("/openapi/v2/job/add", json=camel_data)
        pprint(response.request)
        print(response.json())
        return response.json().get("data")

    def delete(self, job_id):
        # log.info(f"delete job {job_id}")
        response = self._client.post(f"/openapi/v1/job/del/{job_id}")
        pprint(response.request)
        print(response.json())

    def terminate(self, job_id):
        # log.info(f"terminate job {job_id}")
        response = self._client.post(f"/openapi/v1/job/terminate/{job_id}")
        pprint(response.request)
        print(response.json())

    def kill(self, job_id):
        # log.info(f"kill job {job_id}")
        response = self._client.post(f"/openapi/v1/job/kill/{job_id}")
        pprint(response.request)
        print(response.json())

    def log(self, job_id, log_file="STDOUTERR", page=-1, page_size=8192):
        # log.info(f"log job {job_id}")
        response = self._client.get(
            f"/openapi/v1/job/{job_id}/log",
            params={"logFile": log_file, "page": page, "pageSize": page_size},
        )
        pprint(response.request)
        print(response.json().get("data")["log"])
        return response.json().get("data")["log"]

    def create_job(
        self,
        project_id: int,
        name: Optional[str] = None,
        group_id: Optional[int] = 0,
    ):
        # log.info(f"create job {name}")
        # response = self._client.get("/openapi/v1/ak/get")

        # data = {
        #     "userId": response.json().get("data").get("user_id"),
        #     "projectId": project_id,
        #     "name": name,
        #     "bohrGroupId": group_id,
        # }
        # response = self._client.post("/openapi/v1/job/pre_create", json=data)
        # pprint(response.request)
        # print(response.json())
        # return response.json().get("data")
        data = {
            "projectId": project_id,
            "name": name,
            "bohrGroupId": group_id,
        }
        response = self._client.post(f"/openapi/v1/job/create", json=data)
        pprint(response.request)
        print(response.json())
        return response.json().get("data")

    def create_job_group(self, project_id, job_group_name):
        # log.info(f"create job group {job_group_name}")
        response = self._client.post(
            "/openapi/v1/job_group/add",
            json={"name": job_group_name, "projectId": project_id},
        )
        pprint(response.request)
        print(response.json())

    def upload(
        self,
        file_path: str,
        object_key: str,
        token: str,
    ):
        tiefblue = Tiefblue()
        tiefblue.upload_From_file_multi_part(
            object_key=object_key, file_path=file_path, progress_bar=True
        )

    def uploadr(self, work_dir, store_path, token):
        if not work_dir.endswith("/"):
            work_dir = work_dir + "/"
        for root, _, files in os.walk(work_dir):
            for file in files:
                full_path = os.path.join(root, file)
                object_key = full_path.replace(work_dir, store_path)
                self.upload(full_path, object_key, token)

    def download(self, job_id, save_path):
        detail = self.detail(job_id)
        tiefblue = Tiefblue()
        tiefblue.download_from_url(detail["resultUrl"], save_path)


class AsyncJob(AsyncAPIResource):
    pass
